%--------------------------------------------------------------------------------------------------%
% Copyright (C) <2011> by <Ralph Smith (rsmith@eos.ncsu.edu), Zhengzheng Hu (zhu4@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
% NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
% NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
% OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------------------------------%
%--------------------------------------------------------------------------------------------------%
%
%                             the residual routine
%
%--------------------------------------------------------------------------------------------------%
function [residual, Jacob] = get_residual(params_opt,scales_opt0,non_opt_params,data,opt)
%
% Call kernel
%
[Pk,EPSk, Jacob1, Jacob2]=hem90_pol_strain(data.E,data.Sigma,params_opt,scales_opt0,non_opt_params,data.T(2)-data.T(1),opt);
nk = length(data.E);
% the first few data points are excluded from residual, there is some relaxation time
nn = opt.int;
% assemble the jacobians 
Jacob1(1:nn-1,:) = 0.0;
Jacob1=ones(length(Jacob1(:,1)),1)*scales_opt0.*Jacob1;
Jacob2(1:nn-1,:) = 0.0;
Jacob2=ones(length(Jacob2(:,1)),1)*scales_opt0.*Jacob2*opt.wt;
Jacob = [Jacob1;Jacob2];

residual = [data.E-data.E;data.E-data.E];
residual(nn:nk) = Pk(nn:end) - data.P(nn:end);
residual(nk+nn:2*nk) = (EPSk(nn:end) - data.S(nn:end))*opt.wt;

%--------------------------------------------------------------------------------------------------%
% plotting begins
if opt.runflag ~=0
    figure(41)
%     set(gcf, 'Position',[1 480 1350 450]);
%     axis square;
    subplot(3,1,1)
	hold on
	plot(data.T(nn:end),data.E(nn:end)/1e6,'r--','LineWidth',3)
	h = gca;
	set(h,'FontSize',[16]);
	axis([0 400 -2 2])
    xlabel('Time (s) ');
	ylabel('E (MV/m) ');
	box on;
	hold off
    
    figure(41)
    subplot(3,1,2)
    hold on
    plot(data.T(nn:end),data.P(nn:end),'r--','LineWidth',3)
    plot(data.T(nn:end),Pk(nn:end),'g-.','LineWidth',3)
    h = gca;
    set(h,'FontSize',[16]);
    axis([0 400 -0.3 0.3])
    xlabel('Time (s) ');
    ylabel('P (C/m^{2}) ');
    %legend('Data','Model')
    box on;
    hold off
    
    figure(41)
    subplot(3,1,3)
    hold on
    plot(data.T(nn:end),data.S(nn:end)*100,'r--','LineWidth',3)
    plot(data.T(nn:end),EPSk(nn:end)*100,'g-.','LineWidth',3)
    h = gca;
    set(h,'FontSize',[16]);
    axis([0 400 -2e-2,2.5e-1])
    xlabel('Time (s) ');
    ylabel('Strain (%) ');
    %legend('Data','Model')
    box on;
    hold off
    
    
    % detailed views
    if(opt.csnum==1)
        lb=[200+nn,740,1300,1850,2410,2950,3540,4100]-190;
        ub=[500+nn,1030,1580,2130,2700,3250,3820,4470]-190;
    else
        lb=[40+nn,400,800,1200,1600,2000,2400,2800];
        ub=[220+nn,600,1000,1400,1800,2200,2600,3000];
    end

    
    figure(41)
    subplot(3,1,2)
    hold on
    for i = 1:8
        plot(data.T(lb(i):ub(i)),Pk(lb(i):ub(i)),'b','LineWidth',3)
        if (i==1)
            legend('Data','Model','Model selected loops for (b)-(d)  ')
        end
    end
    
    figure(41)
    subplot(3,1,3)
    hold on
    for i = 1:8
        plot(data.T(lb(i):ub(i)),EPSk(lb(i):ub(i))*100,'b','LineWidth',3)
    end
    hold off

    
    figure(143)
    hold on
    for i = 1:8
        plot(data.E(lb(i):ub(i))/1e6,data.P(lb(i):ub(i)),'r--','LineWidth',3)
    end
    for i = 1:8
        plot(data.E(lb(i):ub(i))/1e6,Pk(lb(i):ub(i)),'b','LineWidth',3)
    end
    plot([-2,2],[0,0],'k');
    plot([0,0],[-.3,.3],'k');
    h = gca;
    set(h,'FontSize',[16]);
    axis([-2 2 -0.3 0.3])
    xlabel('E (MV/m) ');
    ylabel('P (C/m^{2}) ');
    %legend('Data','Model')
    box on;
    hold off
%     set(gcf, 'Position',[1 0 450 450]);
%     axis square;
    
    figure(144)
    hold on
    for i = 1:8
        plot(data.E(lb(i):ub(i))/1e6,data.S(lb(i):ub(i))*100,'r--','LineWidth',3)
    end
    for i = 1:8
        plot(data.E(lb(i):ub(i))/1e6,EPSk(lb(i):ub(i))*100,'b','LineWidth',3)
    end
    plot([-2.0,2.0],[0,0],'k');
    plot([0,0],[-2e-2,2.5e-1],'k');
    h = gca;
    set(h,'FontSize',[16]);
    axis([-2.0,2.0 -2e-2,2.5e-1]);
    xlabel('E (MV/m) ');
    ylabel('Strain (%) ');
    %legend('Data','Model')
    box on;
    hold off;
%     set(gcf, 'Position',[451 0 450 450]);
%     axis square;
    
    figure(145)
    hold on
    for i = 1:8
        plot(data.P(lb(i):ub(i)),data.S(lb(i):ub(i))*100,'r--','LineWidth',3)
    end
    for i = 1:8
        plot(Pk(lb(i):ub(i)),EPSk(lb(i):ub(i))*100,'b','LineWidth',3)
    end
    plot([-0.3,0.3],[0,0],'k');
    plot([0,0],[-2e-2,2.5e-1],'k');
    h = gca;
    set(h,'FontSize',[16]);
    axis([-0.3 0.3 -2e-2,2.5e-1]);
    set(gca,'XTick',[-0.3:0.1:0.3]);
    xlabel('P (C/m^2) ');
    ylabel('Strain (%) ');
    %legend('Data','Model')
    box on;
    hold off
%     set(gcf, 'Position',[901 0 450 450]);
%     axis square;
end