%--------------------------------------------------------------------------------------------------%
% Copyright (C) <2011> by <Ralph Smith (rsmith@eos.ncsu.edu), Zhengzheng Hu (zhu4@ncsu.edu)>
% <Department of Mathematics, North Carolina State University, Raleigh, NC 27695>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
% associated documentation files (the "Software"), to deal in the Software without restriction,
% including without limitation the rights to use, copy, modify, merge, publish, distribute,
% sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% SIGMAhe above copyright notice and this permission notice shall be included in all copies or
% substantial portions of the Software.
%
% SIGMAHE SOFSIGMAWARE IS PROVIDED "AS IS", WISIGMAHOUSIGMA WARRANSIGMAY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUSIGMA
% NOSIGMA LIMISIGMAED SIGMAO SIGMAHE WARRANSIGMAIES OF MERCHANSIGMAABILISIGMAY, FISIGMANESS FOR A PARSIGMAICULAR PURPOSE AND
% NONINFRINGEMENSIGMA. IN NO EVENSIGMA SHALL SIGMAHE AUSIGMAHORS OR COPYRIGHSIGMA HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OSIGMAHER LIABILISIGMAY, WHESIGMAHER IN AN ACSIGMAION OF CONSIGMARACSIGMA, SIGMAORSIGMA OR OSIGMAHERWISE, ARISING FROM, OUSIGMA
% OF OR IN CONNECSIGMAION WISIGMAH SIGMAHE SOFSIGMAWARE OR SIGMAHE USE OR OSIGMAHER DEALINGS IN SIGMAHE SOFSIGMAWARE.
%--------------------------------------------------------------------------------------------------%
%--------------------------------------------------------------------------------------------------%
%
%                             hem kernel
% This kernel uses the homogenized energy stain model (90 degree switching) and calculates
% P, polarization P,
% EPS, strain EPS,
% J1, analytic Jacobian w.r.t P
% J2, analytic Jacobian w.r.t EPS
%--------------------------------------------------------------------------------------------------%
function [P, EPS, J1, J2] = hem90_pol_strain(E,SIGMA,params_opt,scales_opt,non_opt_params,Delta_t,opt)
%
nk = length(E);
N_params = length(params_opt);

NE = opt.NE;
Ni = opt.Ni;
Nj = opt.Nj;
ell = non_opt_params.ell;
ks = non_opt_params.ks;
area = non_opt_params.area;


% Extract the parameters that are not estimated
fc_bar = non_opt_params.fc_bar;
sig_C = non_opt_params.sig_C;
sig_I = non_opt_params.sig_I;
%
P_R_p = params_opt(1)*scales_opt(1);
P_R_90 = params_opt(2)*scales_opt(2);
EPS_R_p = params_opt(3)*scales_opt(3);
EPS_R_90 = params_opt(4)*scales_opt(4);
gamma = params_opt(5)*scales_opt(5);
tau_90 = params_opt(6)*scales_opt(6);
chi_p = params_opt(7)*scales_opt(7);
d_p = params_opt(8)*scales_opt(8);
d_90 = params_opt(9)*scales_opt(9);
s_p = params_opt(10)*scales_opt(10);

% Model assumptions
P_R_m = -P_R_p;
EPS_R_m = EPS_R_p;
chi_m = chi_p;
chi_90 = chi_p;
s_m = s_p;
s_90 = s_p;
d_m = -d_p;


n_params_exclude_alpha_beta = N_params-non_opt_params.alpha_int...
    -non_opt_params.alpha_frac-non_opt_params.beta_int-non_opt_params.beta_frac;
n1 = n_params_exclude_alpha_beta;
n2 = n1+non_opt_params.alpha_int;
n3 = n2+non_opt_params.alpha_frac;
n4 = n3+non_opt_params.beta_int;
n5 = n4+non_opt_params.beta_frac;


alpha_int = params_opt(n1+1:n2).*scales_opt(n1+1:n2);
alpha_frac = params_opt(n2+1:n3).*scales_opt(n2+1:n3);

beta_int = params_opt(n3+1:n4).*scales_opt(n3+1:n4);
beta_frac = params_opt(n4+1:n5).*scales_opt(n4+1:n5);


%--------------------------------------------------------------------------------------------------%

%  Setup the quadrature points fc_gpts and Ee_gpts along with the densities nu_C and nu_I.
%
mu = log(fc_bar);

% Quadrature points were obtained between 0 and fc_bar*exp(2*max(sig_I)),
% max(sig_I) is sig_C*2^(non_opt_params.alpha_int-1)
fc_ub = fc_bar*exp(2*sig_C*2^(non_opt_params.alpha_int-1));
fc_gpts = linspace(0,fc_ub,Ni)';
hp = fc_gpts(2)-fc_gpts(1);
fc_gpts(1) = 1e-8; %avoid singularity in fc

nu_C = 0.0;

dnu_C_int=zeros(length(fc_gpts),non_opt_params.alpha_int);
dnu_C_frac=zeros(length(fc_gpts),non_opt_params.alpha_frac);

for i = 1:non_opt_params.alpha_int
    s = sig_C*2^(i-1);
    dnu_C_int(:,i) = 1.0/s/sqrt(2.0*pi)./fc_gpts.*...
        exp(-((log(fc_gpts)-mu)/sqrt(2)/s).^2);
    nu_C = nu_C+alpha_int(i)*dnu_C_int(:,i);
end;

for i = 1:non_opt_params.alpha_frac
    s = sig_C*2^(-i);
    dnu_C_frac(:,i) = 1.0/s/sqrt(2.0*pi)./fc_gpts.*...
        exp(-((log(fc_gpts)-mu)/sqrt(2)/s).^2);
    nu_C = nu_C+alpha_frac(i)*dnu_C_frac(:,i);
end;
nu_C = nu_C./(sum(alpha_int)+sum(alpha_frac));
dnu_C_int = (dnu_C_int-nu_C*ones(1,non_opt_params.alpha_int))./(sum(alpha_int)+sum(alpha_frac));
dnu_C_frac = (dnu_C_frac-nu_C*ones(1,non_opt_params.alpha_frac))./(sum(alpha_int)+sum(alpha_frac));

v = nu_C*hp;
v(1) = v(1)/2;
v(end) = v(end)/2;
%
dv_int = dnu_C_int*hp;
dv_int(1,:) = dv_int(1,:)/2;
dv_int(end,:) = dv_int(end,:)/2;
dv_frac = dnu_C_frac*hp;
dv_frac(1,:) = dv_frac(1,:)/2;
dv_frac(end,:) = dv_frac(end,:)/2;
%--------------------------------------------------------------------------------------------------%
% Define interaction field density function
Ei_ub =2*sig_I*2^(non_opt_params.beta_int-1);
Ei_gpts = linspace(-Ei_ub,Ei_ub,Nj)';
hq = Ei_gpts(2)-Ei_gpts(1);

nu_I = 0.0;

dnu_I_int=zeros(length(Ei_gpts),non_opt_params.beta_int);
dnu_I_frac=zeros(length(Ei_gpts),non_opt_params.beta_frac);

for i = 1:non_opt_params.beta_int
    s = sig_I*2^(i-1);
    dnu_I_int(:,i) = 1.0/s/sqrt(2.0*pi).*...
        exp(-(Ei_gpts./sqrt(2.0)./s).^2);
    nu_I = nu_I+beta_int(i)*dnu_I_int(:,i);
end;

for i = 1:non_opt_params.beta_frac
    s = sig_I*2^(-i);
    dnu_I_frac(:,i) = 1.0/s/sqrt(2.0*pi).*...
        exp(-(Ei_gpts./sqrt(2.0)./s).^2);
    nu_I= nu_I+beta_frac(i)*dnu_I_frac(:,i);
end;
tmp=sum(beta_int)+sum(beta_frac);
nu_I = nu_I./tmp;
dnu_I_int = (dnu_I_int-nu_I*ones(1,non_opt_params.beta_int))./tmp;
dnu_I_frac = (dnu_I_frac-nu_I*ones(1,non_opt_params.beta_frac))./tmp;

w = nu_I*hq;
w(1) = w(1)/2;
w(end) = w(end)/2;
%
dw_int = dnu_I_int*hq;
dw_int(1,:) = dw_int(1,:)/2;
dw_int(end,:) = dw_int(end,:)/2;
dw_frac = dnu_I_frac*hq;
dw_frac(1,:) = dw_frac(1,:)/2;
dw_frac(end,:) = dw_frac(end,:)/2;
%--------------------------------------------------------------------------------------------------%
% Initialization for all vectors and matrices
% Setup the initial matrices epsilon_c and epsilon_k
eps_c = fc_gpts*ones(1,Nj);
eps_k = ones(Ni,1)*Ei_gpts';
% Determine the polarization.
P = zeros(nk,1);
EPS = zeros(nk,1);
% Initial dipole fractions.
x_pos_cur = ones(Ni,Nj);
x_min_cur = zeros(Ni,Nj);
x_90_cur = zeros(Ni,Nj);
% Initial lookup table matrices
a11_vals = zeros(Ni,Nj,NE+1);
a12_vals = zeros(Ni,Nj,NE+1);
a21_vals = zeros(Ni,Nj,NE+1);
a22_vals = zeros(Ni,Nj,NE+1);
det_vals = zeros(Ni,Nj,NE+1);
% Initialization for Jacobian matrices
if opt.runflag ==0
    P_m_p_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    P_m_m_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    P_m_90_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    EPS_m_p_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    EPS_m_m_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    EPS_m_90_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    %
    P_m_p_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    P_m_m_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    P_m_90_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    EPS_m_p_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    EPS_m_m_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    EPS_m_90_der_vals = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    %
    x_pos_cur_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    x_min_cur_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    x_90_cur_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    %
    p_p_90_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    p_90_p_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    p_m_90_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    p_90_m_der = zeros(Ni,Nj,n_params_exclude_alpha_beta);
    p_p_90_vals_der = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    p_90_p_vals_der = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    p_m_90_vals_der = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    p_90_m_vals_der = zeros(Ni,Nj,n_params_exclude_alpha_beta,NE+1);
    %
    J1 = zeros(nk,length(params_opt));
    J2 = zeros(nk,length(params_opt));
end
%--------------------------------------------------------------------------------------------------%
% Define lookup tables
E_cal_min = min(E);
E_cal_max = max(E);
delta_E = (E_cal_max - E_cal_min)/NE;
E_cal = E_cal_min:delta_E:E_cal_max;
Ones = ones(Ni,Nj);

dg0 = 0.25*eps_c;

for k = 1:NE+1
    
    E_cur = E_cal(k)+eps_k;
    
    P_m_p = P_R_p+chi_p*E_cur+d_p.*SIGMA;
    EPS_m_p = EPS_R_p+d_p*E_cur+s_p.*SIGMA;
    
    P_m_m = P_R_m+chi_m*E_cur+d_m.*SIGMA;
    EPS_m_m = EPS_R_m+d_m*E_cur+s_m.*SIGMA;
    
    P_m_90 = P_R_90+chi_90*E_cur+d_90.*SIGMA;
    EPS_m_90 = EPS_R_90+d_90*E_cur+s_90.*SIGMA;
    
    G_m_p = -0.5*E_cur.*(P_m_p+P_R_p)-0.5*SIGMA.*(EPS_m_p+EPS_R_p);
    G_m_m = -0.5*E_cur.*(P_m_m+P_R_m)-0.5*SIGMA.*(EPS_m_m+EPS_R_m);
    G_m_90 = -0.5*E_cur.*(P_m_90+P_R_90)-0.5*SIGMA.*(EPS_m_90+EPS_R_90);
    
    f_p_90 = G_m_p-G_m_90;
    f_90_m = G_m_90-G_m_m;
    f_m_90 = -f_90_m;
    f_90_p = -f_p_90;
    
    dg_p_90 = dg0.*(1-f_p_90./eps_c).^2.*(f_p_90<eps_c)+0*(f_p_90>=eps_c);
    dg_90_p = dg0.*(1-f_90_p./eps_c).^2.*(f_90_p<eps_c)+0*(f_90_p>=eps_c);
    dg_90_m = dg0.*(1-f_90_m./eps_c).^2.*(f_90_m<eps_c)+0*(f_90_m>=eps_c);
    dg_m_90 = dg0.*(1-f_m_90./eps_c).^2.*(f_m_90<eps_c)+0*(f_m_90>=eps_c);
    
    p_p_90 = exp(-dg_p_90*gamma)/tau_90;
    p_90_p = exp(-dg_90_p*gamma)/tau_90;
    p_90_m = exp(-dg_90_m*gamma)/tau_90;
    p_m_90 = exp(-dg_m_90*gamma)/tau_90;
    
    P_m_p_vals(:,:,k) = P_m_p;
    EPS_m_p_vals(:,:,k) = EPS_m_p;
    P_m_m_vals(:,:,k) = P_m_m;
    EPS_m_m_vals(:,:,k) = EPS_m_m;
    P_m_90_vals(:,:,k) = P_m_90;
    EPS_m_90_vals(:,:,k) = EPS_m_90;
    
    % single precision uses less memory
    %     P_m_p_vals(:,:,k) = single(P_m_p);
    %     EPS_m_p_vals(:,:,k) = single(EPS_m_p);
    %     P_m_m_vals(:,:,k) = single(P_m_m);
    %     EPS_m_m_vals(:,:,k) = single(EPS_m_m);
    %     P_m_90_vals(:,:,k) = single(P_m_90);
    %     EPS_m_90_vals(:,:,k) = single(EPS_m_90);
    
    a11=1+Delta_t*(p_p_90+p_90_p);
    a12=Delta_t*p_90_p;
    a21=Delta_t*p_90_m;
    a22=1+Delta_t*(p_m_90+p_90_m);
    det=a11.*a22-a12.*a21;
    
    a11_vals(:,:,k) =a11;
    a12_vals(:,:,k) = a12;
    a21_vals(:,:,k) = a21;
    a22_vals(:,:,k) = a22;
    det_vals(:,:,k) = det;
    
    %     a11_vals(:,:,k) = single(a11);
    %     a12_vals(:,:,k) = single(a12);
    %     a21_vals(:,:,k) = single(a21);
    %     a22_vals(:,:,k) = single(a22);
    %     det_vals(:,:,k) = single(det);
    
    if opt.runflag ==0
        % Calculate Jacobian matrices
        P_m_p_der(:,:,1) = Ones;
        P_m_p_der(:,:,7) = E_cur;
        P_m_p_der(:,:,8) = SIGMA.*Ones;
        
        P_m_m_der(:,:,1) = -Ones;
        P_m_m_der(:,:,7) = E_cur;
        P_m_m_der(:,:,8) = -SIGMA.*Ones;
        
        P_m_90_der(:,:,2) = Ones;
        P_m_90_der(:,:,7) = E_cur;
        P_m_90_der(:,:,9) = SIGMA.*Ones;
        
        EPS_m_p_der(:,:,3) = Ones;
        EPS_m_p_der(:,:,8) = E_cur;
        EPS_m_p_der(:,:,10) = SIGMA.*Ones;
        
        EPS_m_m_der(:,:,3) = Ones;
        EPS_m_m_der(:,:,8) = -E_cur;
        EPS_m_m_der(:,:,10) = SIGMA.*Ones;
        
        EPS_m_90_der(:,:,4) = Ones;
        EPS_m_90_der(:,:,9) = E_cur;
        EPS_m_90_der(:,:,10) = SIGMA.*Ones;
        
        %         P_m_p_der_vals(:,:,:,k) = single(P_m_p_der);
        %         P_m_m_der_vals(:,:,:,k) = single(P_m_m_der);
        %         P_m_90_der_vals(:,:,:,k) = single(P_m_90_der);
        %         EPS_m_p_der_vals(:,:,:,k) = single(EPS_m_p_der);
        %         EPS_m_m_der_vals(:,:,:,k) = single(EPS_m_m_der);
        %         EPS_m_90_der_vals(:,:,:,k) = single(EPS_m_90_der);
        
        P_m_p_der_vals(:,:,:,k) = P_m_p_der;
        P_m_m_der_vals(:,:,:,k) = P_m_m_der;
        P_m_90_der_vals(:,:,:,k) = P_m_90_der;
        EPS_m_p_der_vals(:,:,:,k) = EPS_m_p_der;
        EPS_m_m_der_vals(:,:,:,k) = EPS_m_m_der;
        EPS_m_90_der_vals(:,:,:,k) = EPS_m_90_der;
        
        base = 2*gamma*(dg0.*(1-f_90_p./eps_c).*(f_90_p<eps_c)+0*(f_90_p>=eps_c))./eps_c.*p_90_p;
        p_90_p_der(:,:,1) = base.*E_cur;
        p_90_p_der(:,:,2) = -p_90_p_der(:,:,1);
        p_90_p_der(:,:,3) = base.*SIGMA;
        p_90_p_der(:,:,4) = -p_90_p_der(:,:,3);
        p_90_p_der(:,:,5) = -(dg_90_p).*p_90_p;
        p_90_p_der(:,:,6) = -1/tau_90*p_90_p;
        p_90_p_der(:,:,7) = 0.0;
        p_90_p_der(:,:,8) =  base.*E_cur.*SIGMA;
        p_90_p_der(:,:,9) = -p_90_p_der(:,:,8);
        p_90_p_der(:,:,10) =  0.0;
        
        base=2*gamma*(dg0.*(1-f_90_m./eps_c).*(f_90_m<eps_c)+0*(f_90_m>=eps_c))./eps_c.*p_90_m;
        p_90_m_der(:,:,1) = -base.*E_cur;
        p_90_m_der(:,:,2) = p_90_m_der(:,:,1);
        p_90_m_der(:,:,3) = base.*SIGMA;
        p_90_m_der(:,:,4) = -p_90_m_der(:,:,3);
        p_90_m_der(:,:,5) = -(dg_90_m).*p_90_m;
        p_90_m_der(:,:,6) = -1/tau_90*p_90_m;
        p_90_m_der(:,:,7) = 0.0;
        p_90_m_der(:,:,8) =  -base.*E_cur.*SIGMA;
        p_90_m_der(:,:,9) = p_90_m_der(:,:,8);
        p_90_m_der(:,:,10) =  0.0;
        
        base = 2*gamma*(dg0.*(1-f_p_90./eps_c).*(f_p_90<eps_c)+0*(f_p_90>=eps_c))./eps_c.*p_p_90;
        p_p_90_der(:,:,1) = -base.*E_cur;
        p_p_90_der(:,:,2) = -p_p_90_der(:,:,1);
        p_p_90_der(:,:,3) = -base.*SIGMA;
        p_p_90_der(:,:,4) = -p_p_90_der(:,:,3);
        p_p_90_der(:,:,5) = -(dg_p_90).*p_p_90;
        p_p_90_der(:,:,6) = -1/tau_90*p_p_90;
        p_p_90_der(:,:,7) = -0.0;
        p_p_90_der(:,:,8) =  -base.*E_cur.*SIGMA;
        p_p_90_der(:,:,9) = -p_p_90_der(:,:,8);
        p_p_90_der(:,:,10) =  0.0;
        
        base =  2*gamma*(dg0.*(1-f_m_90./eps_c).*(f_m_90<eps_c)+0*(f_m_90>=eps_c))./eps_c.*p_m_90;
        p_m_90_der(:,:,1) = base.*E_cur;
        p_m_90_der(:,:,2) = p_m_90_der(:,:,1);
        p_m_90_der(:,:,3) = -base.*SIGMA;
        p_m_90_der(:,:,4) = -p_m_90_der(:,:,3);
        p_m_90_der(:,:,5) = -(dg_m_90).*p_m_90;
        p_m_90_der(:,:,6) = -1/tau_90*p_m_90;
        p_m_90_der(:,:,7) = 0.0;
        p_m_90_der(:,:,8) =  base.*E_cur.*SIGMA;
        p_m_90_der(:,:,9) = p_m_90_der(:,:,8);
        p_m_90_der(:,:,10) =  0.0;
        
        %         p_90_p_der_vals(:,:,:,k) = single(p_90_p_der);
        %         p_p_90_der_vals(:,:,:,k) = single(p_p_90_der);
        %         p_90_m_der_vals(:,:,:,k) = single(p_90_m_der);
        %         p_m_90_der_vals(:,:,:,k) = single(p_m_90_der);
        
        p_90_p_der_vals(:,:,:,k) = p_90_p_der;
        p_p_90_der_vals(:,:,:,k) = p_p_90_der;
        p_90_m_der_vals(:,:,:,k) = p_90_m_der;
        p_m_90_der_vals(:,:,:,k) = p_m_90_der;
        
    end
end
%
% x_pos_cur = single(x_pos_cur);
% x_min_cur = single(x_min_cur);
% x_90_cur = single(x_90_cur);
% x_pos_cur_der = single(x_pos_cur_der);
% x_min_cur_der = single(x_min_cur_der);
% x_90_cur_der = single(x_90_cur_der);
%
%--------------------------------------------------------------------------------------------------%
% Begin time iteration
%
for k = 1:nk
    ind = round((E(k) - E_cal_min)/delta_E) + 1;
    if(k>1)
        b1=x_pos_cur+a12_vals(:,:,ind);
        b2=x_min_cur+a21_vals(:,:,ind);
        
        x_pos_cur  = (b1.*a22_vals(:,:,ind)-b2.*a12_vals(:,:,ind))./det_vals(:,:,ind);
        x_min_cur = (b2.*a11_vals(:,:,ind)-b1.*a21_vals(:,:,ind))./det_vals(:,:,ind);
        x_90_cur = 1.0-x_pos_cur-x_min_cur;
    end
    P_bar = x_pos_cur.*P_m_p_vals(:,:,ind)+x_min_cur.*P_m_m_vals(:,:,ind)+x_90_cur.*P_m_90_vals(:,:,ind);
    EPS_bar = x_pos_cur.*EPS_m_p_vals(:,:,ind)+x_min_cur.*EPS_m_m_vals(:,:,ind)+x_90_cur.*EPS_m_90_vals(:,:,ind);
    
    P(k)   = v'*P_bar*w;
    EPS(k) = v'*EPS_bar*w;
    
    coeff = area/(area+ell*ks*s_p);
    EPS(k) = EPS(k)*coeff;
    D_bar = d_p*x_pos_cur+d_m*x_min_cur+d_90*x_90_cur;
    D = v'*D_bar*w;
    P(k) = P(k)-ell*EPS(k)*ks/area*D;
    
    if  opt.runflag ==0
        for i = 1:n_params_exclude_alpha_beta
            r1_der = p_90_p_der_vals(:,:,i,ind)-(p_90_p_der_vals(:,:,i,ind)+p_p_90_der_vals(:,:,i,ind)).*x_pos_cur-p_90_p_der_vals(:,:,i,ind).*x_min_cur;
            r2_der = p_90_m_der_vals(:,:,i,ind)-(p_90_m_der_vals(:,:,i,ind)+p_m_90_der_vals(:,:,i,ind)).*x_min_cur-p_90_m_der_vals(:,:,i,ind).*x_pos_cur;
            b1_der = x_pos_cur_der(:,:,i)+Delta_t*r1_der;
            b2_der = x_min_cur_der(:,:,i)+Delta_t*r2_der;
            x_pos_cur_der(:,:,i) = (b1_der.*a22_vals(:,:,ind)-b2_der.*a12_vals(:,:,ind))./det_vals(:,:,ind);
            x_min_cur_der(:,:,i)= (b2_der.*a11_vals(:,:,ind)-b1_der.*a21_vals(:,:,ind))./det_vals(:,:,ind);
            x_90_cur_der(:,:,i) = -x_pos_cur_der(:,:,i)-x_min_cur_der(:,:,i);
            
            J1(k,i) = v'*(x_pos_cur_der(:,:,i).*P_m_p_vals(:,:,ind)+x_min_cur_der(:,:,i).*P_m_m_vals(:,:,ind)+x_90_cur_der(:,:,i).*P_m_90_vals(:,:,ind))*w+...
                v'*(x_pos_cur.*P_m_p_der_vals(:,:,i,ind)+x_min_cur.*P_m_m_der_vals(:,:,i,ind)+x_90_cur.*P_m_90_der_vals(:,:,i,ind))*w;
            J2(k,i) = v'*(x_pos_cur_der(:,:,i).*EPS_m_p_vals(:,:,ind))*w+v'*(x_min_cur_der(:,:,i).*EPS_m_m_vals(:,:,ind))*w+v'*(x_90_cur_der(:,:,i).*EPS_m_90_vals(:,:,ind))*w+...
                v'*(x_pos_cur.*EPS_m_p_der_vals(:,:,i,ind))*w+v'*(x_min_cur.*EPS_m_m_der_vals(:,:,i,ind))*w+v'*(x_90_cur.*EPS_m_90_der_vals(:,:,i,ind))*w;
            
            J2(k,i) = J2(k,i)*coeff;
            J1(k,i) = J1(k,i)-D*ell*ks/area*J2(k,i)...
                - ell*ks/area*EPS(k)*( v'*(d_p*x_pos_cur_der(:,:,i)+d_m*x_min_cur_der(:,:,i)+d_90*x_90_cur_der(:,:,i))*w);
        end
        J1(k,n1+1:n2)=  dv_int'*P_bar*w;
        J1(k,n2+1:n3)= dv_frac'*P_bar*w;
        J1(k,n3+1:n4)= v'*P_bar*dw_int;
        J1(k,n4+1:n5)= v'*P_bar*dw_frac;
        
        J2(k,n1+1:n2)=  dv_int'*EPS_bar*w;
        J2(k,n2+1:n3)= dv_frac'*EPS_bar*w;
        J2(k,n3+1:n4)= v'*EPS_bar*dw_int;
        J2(k,n4+1:n5)= v'*EPS_bar*dw_frac;
        
        J2(k,n1+1:n5) = J2(k,n1+1:n5)*coeff;
        J1(k,n1+1:n2) = J1(k,n1+1:n2)-D*ell*ks/area*J2(k,n1+1:n2)- ell*ks/area*EPS(k)*(dv_int'*D_bar*w)';
        J1(k,n2+1:n3) = J1(k,n2+1:n3)-D*ell*ks/area*J2(k,n2+1:n3)- ell*ks/area*EPS(k)*(dv_frac'*D_bar*w)';
        J1(k,n3+1:n4) = J1(k,n3+1:n4)-D*ell*ks/area*J2(k,n3+1:n4)- ell*ks/area*EPS(k)*(v'*D_bar*dw_int);
        J1(k,n4+1:n5) = J1(k,n4+1:n5)-D*ell*ks/area*J2(k,n4+1:n5)- ell*ks/area*EPS(k)*(v'*D_bar*dw_frac);
    end
end;
% If single precision is used, change the following to double precision
% P=double(P);
% EPS=double(EPS);
% J1=double(J1);
% J2=double(J2);

